@@ -5,7 +5,7 @@ use warnings;
 
 use File::Temp ();
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 {
     package Process::Pipeline::Process;
@@ -85,6 +85,10 @@ our $VERSION = '0.02';
                 $found->{status} = Process::Status->new($save);
             }
         }
+        if (my $filename = delete $self->{_filename}) {
+            local $!;
+            unlink $filename;
+        }
         $self;
     }
 }
@@ -116,19 +120,19 @@ sub start {
         $_->close for @close;
     };
 
-    my ($main_out_fh, $main_out_name);
+    my ($main_out_fh, $main_out_filename);
     my $result = Process::Pipeline::Result->new;
     for my $i (0..$n) {
         my $process = $self->{process}[$i];
         if ($i == $n && !$process->set->{">"} && !$process->set->{">>"}) {
-            ($main_out_fh, $main_out_name) = File::Temp::tempfile(UNLINK => 0);
+            ($main_out_fh, $main_out_filename) = File::Temp::tempfile(UNLINK => 0);
         }
         my $pid = fork;
         die "fork: $!" unless defined $pid;
         if ($pid == 0) {
-            if ($main_out_name) {
+            if ($main_out_filename) {
                 close $main_out_fh;
-                open STDOUT, ">>", $main_out_name or die $!;
+                open STDOUT, ">>", $main_out_filename or die $!;
             }
             $close->($i);
             my $read  = $i - 1 >= 0 ? $pipe[$i - 1] : undef;
@@ -181,9 +185,8 @@ sub start {
         ));
     }
     $_->close for map { @$_ } @pipe;
-    if ($main_out_fh) {
-        select undef, undef, undef, 0.01;
-        unlink $main_out_name;
+    if ($main_out_filename) {
+        $result->{_filename} = $main_out_filename;
         $result->fh($main_out_fh);
     }
     $result->wait unless $option{async};
@@ -214,8 +217,8 @@ In perl5:
     ->push(sub { my $p = shift; $p->cmd("grep", "-v", "127.0.0.1") })
     ->push(sub { my $p = shift; $p->cmd("grep", "-c", "POST"       });
 
-  my $r = $pipeline->start;
-  if ($r->is_success) {
+  my $result = $pipeline->start;
+  if ($result->is_success) {
      my $fh = $result->fh; # output filehandle of $pipeline
      say <$fh>;
   }
@@ -228,8 +231,8 @@ In perl5 with DSL:
                  proc { "grep", "-v", "127.0.0.1" }
                  proc { "grep", "-c", "POST"      };
 
-  my $r = $pipeline->start;
-  if ($r->is_success) {
+  my $result = $pipeline->start;
+  if ($result->is_success) {
      my $fh = $result->fh; # output filehandle of $pipeline
      say <$fh>;
   }
